'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.behead = behead;

var _remark = require('remark');

var _remark2 = _interopRequireDefault(_remark);

var _defop = require('defop');

var _defop2 = _interopRequireDefault(_defop);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 * Constants
 */
/**
 * Behead is a [remark](https://github.com/wooorm/remark) plugin to 
 * increase and decrease the weight of markdown headings. Passing a 
 * negative value to the weight option will decrease the heading weight.
 * Passing a positive value to the weight option will increase the heading 
 * weight
 *
 * [![Build Status](https://img.shields.io/circleci/project/mrzmmr/remark-behead/master.svg)](https://circleci.com/gh/mrzmmr/remark-behead/tree/master)
 *
 * [![Coverage Status](https://img.shields.io/coveralls/mrzmmr/remark-behead/master.svg)](https://coveralls.io/github/mrzmmr/remark-behead?branch=master)
 *
 * ## install
 *
 * ```
 * npm install --save remark-behead
 * ```
 *
 * @module behead
 * @version 1.5.1
 * @author mrzmmr
 */

/*
 * Imports
 */
var MINWEIGHT = 6;
var MAXWEIGHT = 1;

/**
 * @property {Boolean} preserve - Defaults to true
 * @property {String} before    - Defaults to null
 * @property {String} after     - Defaults to null
 * @property {Number} weight    - Defaults to 0
 */
var options = {
  preserve: true,
  between: null,
  before: null,
  after: null,
  weight: 0
};

module.exports = function plugin(processor) {
  var opts = arguments.length <= 1 || arguments[1] === undefined ? options : arguments[1];

  opts = (0, _defop2.default)(opts, options);

  var switched = false;

  return function (ast, file) {
    return ast.children = ast.children.map(function (node) {

      if (!opts.between && !opts.before && !opts.after) {
        return behead(node, opts);
      }

      /**
       * Manipulates heading nodes after but not including the given 
       * string. _**Note:** When using this option, behead will start 
       * working after the first occurrence of the given string._
       *
       * @name options.after
       *
       * @example
       * remark.use(behead, {weight: 1, after: '# After this'})
       *   .process('# After this\n## Hello\n## World')
       *
       *   => '# After this\n# Hello\n# World\n'
       */
      if (opts.after) {
        if (switched) {
          return behead(node, opts);
        } else {
          if (_remark2.default.stringify(node) === opts.after) {
            switched = true;
          }
          return node;
        }
      }

      /**
       * Manipulates heading nodes before but not including the given 
       * string. _**Note:** When using this option, behead will stop 
       * working at the first occurrence of the given string._
       *
       * @name options.before
       *
       * @example
       * remark.use(behead, {weight: 1, before: '# Before this'})
       *   .process('# Hello\n# World\n# Before this')
       *
       *   => '## Hello\n## World\n# Before this\n'
       */
      if (opts.before) {
        if (switched) {
          return node;
        } else {
          if (_remark2.default.stringify(node) === opts.before) {
            switched = true;
            return node;
          } else {
            return behead(node, opts);
          }
        }
      }

      /**
       * Manipulates hading nodes between but not including the two given 
       * strings, starting with options.between[0] and ending with
       * options.between[1].
       *
       * @name options.between
       *
       * @example
       * remark(behead, {weight: 1, between: ['# Hello', '# World']})
       *   .process('# Hello\n# Between\n# World')
       *
       *   => '# Hello\n## Between\n# World\n'
       */
      if (opts.between) {
        if (switched) {

          if (_remark2.default.stringify(node) === opts.between[1]) {
            switched = false;
            return node;
          }

          return behead(node, opts);
        }

        if (_remark2.default.stringify(node) === opts.between[0]) {
          switched = true;
          return node;
        }

        return node;
      }
    });
  };
};

function behead(node, opts) {

  if (node.type && node.type === 'heading') {

    if (opts.weight < 0) {
      node.depth += Math.abs(opts.weight);

      if (node.depth > MINWEIGHT) {

        if (opts.preserve) {
          node.depth = MINWEIGHT;
        } else {
          node.depth = 0;
        }
      }
    } else {
      node.depth -= opts.weight;

      if (node.depth < MAXWEIGHT) {
        node.depth = MAXWEIGHT;
      }
    }
  }
  return node;
}